import os
import shutil
import test.support
import unittest

from aletheia.aletheia import Aletheia


class AletheiaTestCase(unittest.TestCase):

    SCRATCH = "/tmp/aletheia-tests"

    def setUp(self):

        self.env = test.support.EnvironmentVarGuard()
        self.env["HOME"] = self.SCRATCH

        os.makedirs(self.env["HOME"], exist_ok=True)

    def tearDown(self):
        shutil.rmtree(self.SCRATCH, ignore_errors=True)

    def test___init___defaults(self):

        with self.env:
            aletheia = Aletheia()

        home = "{}/.config/aletheia".format(self.SCRATCH)
        self.assertEqual(
            aletheia.private_key_path, "{}/aletheia.pem".format(home))
        self.assertEqual(
            aletheia.public_key_path, "{}/aletheia.pub".format(home))
        self.assertEqual(
            aletheia.public_key_cache, "{}/public-keys".format(home))

    def test___init___with_values(self):

        with self.env:
            aletheia = Aletheia(
                private_key_path="alpha",
                public_key_path="bravo",
                cache_dir="charlie"
            )

        self.assertEqual(aletheia.private_key_path, "alpha")
        self.assertEqual(aletheia.public_key_path, "bravo")
        self.assertEqual(aletheia.public_key_cache, "charlie")

    def test_generate(self):

        with self.env:
            Aletheia().generate()

        home = os.path.join(self.SCRATCH, ".config", "aletheia")
        self.assertTrue(os.path.exists(os.path.join(home, "aletheia.pem")))
        self.assertTrue(os.path.exists(os.path.join(home, "aletheia.pub")))

        with open(os.path.join(home, "aletheia.pem")) as f:
            self.assertIn("BEGIN", f.read())

        with open(os.path.join(home, "aletheia.pub")) as f:
            self.assertIn("BEGIN", f.read())

    def test_sign_file_doesnt_exist(self):

        with self.env:
            aletheia = Aletheia()

        self.assertRaises(
            FileNotFoundError, aletheia.sign, "/dev/null/nowhere", "")

    def test_verify_file_doesnt_exist(self):

        with self.env:
            aletheia = Aletheia()

        self.assertRaises(
            FileNotFoundError, aletheia.verify, "/dev/null/nowhere")

    def test__get_private_key_in_environment(self):

        private_key_path = os.path.join(
            os.path.dirname(__file__), "data", "key.pem")
        with open(private_key_path) as f:
            self.env["ALETHEIA_PRIVATE_KEY"] = f.read()

        with self.env:
            aletheia = Aletheia()
            self.assertIsNotNone(aletheia._get_private_key())

    def test__get_private_key_in_file(self):
        with self.env:
            aletheia = Aletheia(private_key_path=os.path.join(
                os.path.dirname(__file__), "data", "key.pem"))
            self.assertIsNotNone(aletheia._get_private_key())

    def test__get_private_key_doesnt_exist(self):
        with self.env:
            self.assertRaises(RuntimeError, Aletheia()._get_private_key)



import json

import piexif
import PIL.Image

from ...exceptions import UnparseableFileError
from ..base import File


class JpegFile(File):

    SUPPORTED_TYPES = ("image/jpeg",)

    def get_raw_data(self):
        with PIL.Image.open(self.source) as im:
            return im.tobytes()

    def sign(self, private_key, public_key_url):
        """
        Use Pillow to capture the raw image data, generate a signature from it,
        and then use piexif to write said signature + where to find the public
        key to the image metadata in the following format:

          {"version": int, "public-key": url, "signature": signature}

        :param private_key     key  The private key used for signing
        :param public_key_url  str  The URL where you're storing the public key
        """

        signature = self.generate_signature(private_key)

        self.logger.debug("Signature generated: %s", signature)

        payload = self.generate_payload(public_key_url, signature)

        exif = piexif.load(self.source)
        exif["0th"][piexif.ImageIFD.HostComputer] = payload
        piexif.insert(piexif.dump(exif), self.source)

    def verify(self):
        """
        Attempt to verify the origin of an image by checking its local
        signature against the public key listed in the file.
        :return: boolean  ``True`` if verified, `False`` if not.
        """

        exif = piexif.load(self.source)

        try:
            data = json.loads(
                exif["0th"][piexif.ImageIFD.HostComputer].decode("utf-8"))
            key_url = data["public-key"]
            signature = data["signature"]
        except (KeyError, json.JSONDecodeError):
            self.logger.warning("Invalid format, or no signature found")
            raise UnparseableFileError()

        self.logger.debug("Signature found: %s", signature)

        return self.verify_signature(key_url, signature)



import os

from aletheia.file_types.base import File
from aletheia.file_types import JpegFile, Mp3File, Mp4File

from ..base import TestCase


class FileTestCase(TestCase):

    def test_build_jpg(self):
        self.assertIsInstance(
            File.build(os.path.join(self.DATA, "test.jpg"), self.SCRATCH),
            JpegFile
        )

    def test_build_mp3(self):
        self.assertIsInstance(
            File.build(os.path.join(self.DATA, "test.mp3"), self.SCRATCH),
            Mp3File
        )

    def test_get_subclasses(self):
        self.assertEqual(
            set(File.get_subclasses()),
            {JpegFile, Mp3File, Mp4File}
        )



#
#   $ aletheia generate
#   $ aletheia sign /path/to/file public-key-url
#   $ aletheia verify /path/to/file
#

import argparse
import os

from cryptography.exceptions import InvalidSignature
from termcolor import cprint

from aletheia.aletheia import Aletheia
from aletheia.exceptions import (
    InvalidURLError,
    PublicKeyNotExistsError,
    UnknownFileTypeError,
    UnparseableFileError
)
from aletheia.utils import generate, sign, verify


class Command:

    def __init__(self):

        self.parser = argparse.ArgumentParser(prog="aletheia")
        self.parser.set_defaults(func=self.parser.print_help)
        subparsers = self.parser.add_subparsers(dest="subcommand")

        subparsers.add_parser(
            "generate",
            help="Generate a public/private key pair for use in signing & 3rd "
                 "party verification. (Do this first)"
        )

        parser_sign = subparsers.add_parser("sign", help="Sign a file")
        parser_sign.add_argument("path")
        parser_sign.add_argument(
            "url", nargs="?", default=os.getenv("ALETHEIA_PUBLIC_KEY_URL"))

        parser_verify = subparsers.add_parser(
            "verify", help="Verify the origin of a file")
        parser_verify.add_argument("path")

    @classmethod
    def run(cls):

        instance = cls()

        args = instance.parser.parse_args()

        if args.subcommand:
            return getattr(instance, args.subcommand)(args)

        instance.parser.print_help()
        return 0

    @classmethod
    def generate(cls, *args):

        private = Aletheia().private_key_path
        if os.path.exists(private):
            cprint(
                "It looks like you already have a key setup at {}.\n"
                "Exiting prematurely just to be safe.".format(private),
                "yellow"
            )
            return 1

        cprint("\n  🔑  Generating private/public key pair...", "green")
        generate()
        cprint("""
            All finished!

            You now have two files: aletheia.pem (your private key) and
            aletheia.pub (your public key).  Keep the former private, and share
            the latter far-and-wide.  Importantly, place your public key at a
            publicly accessible URL so that when you sign a file with your
            private key, it can be verified by reading the public key at that
            URL.
        """.replace("          ", ""), "green")

    @classmethod
    def sign(cls, args):

        if not args.url:
            cprint(
                "\n  ✖️  You must specify the public key URL either in the "
                "environment as \n  ALETHEIA_PUBLIC_KEY_URL or on the command "
                "line as the second argument.\n",
                "red"
            )
            return 3

        try:
            sign(args.path, args.url)
        except FileNotFoundError:
            cprint(
                "\n  ✖️  Aletheia can't find that file\n",
                "red"
            )
            return 1
        except UnknownFileTypeError:
            cprint(
                "\n  ✖️  Aletheia doesn't know how to sign that file type\n",
                "red"
            )
            return 2
        template = "\n  ✔  {} was signed with your private key\n"
        cprint(template.format(args.path), "green")

        return 0

    @classmethod
    def verify(cls, args):

        try:
            domain = verify(args.path)
        except FileNotFoundError:
            cprint(
                "\n  ✖️  Aletheia can't find that file\n",
                "red"
            )
            return 1
        except UnknownFileTypeError:
            cprint(
                "\n  ✖️  Aletheia doesn't recognise that file type\n",
                "red"
            )
            return 2
        except UnparseableFileError:
            cprint(
                "\n  ✖️  Aletheia can't find a signature in that file\n",
                "red"
            )
            return 3
        except InvalidURLError:
            cprint(
                "\n  ✖️  The public key URL in the file provided is invalid\n",
                "red"
            )
            return 4
        except PublicKeyNotExistsError:
            cprint(
                "\n  ✖️  The specified URL does not contain a public key\n",
                "red"
            )
            return 5
        except InvalidSignature:
            cprint("\n  ✖️  There's something wrong with that file\n", "red")
            return 6

        template = "\n  ✔  The file is verified as having originated at {}\n"
        cprint(template.format(domain), "green")

        return 0


if __name__ == "__main__":
    Command.run()



import json

import mutagen

from ...exceptions import UnparseableFileError
from ..base import FFMpegFile


class Mp4File(FFMpegFile):

    SCHEMA_VERSION = 1
    SUPPORTED_TYPES = ("video/mp4",)

    def sign(self, private_key, public_key_url):

        signature = self.generate_signature(private_key)

        self.logger.debug("Signature generated: %s", signature)

        payload = self.generate_payload(public_key_url, signature)

        mp4 = mutagen.File(self.source)
        mp4["\xa9too"] = payload  # We use the "Encoded by" tag
        mp4.save()

    def verify(self):

        mp4 = mutagen.File(self.source)

        try:

            payload = json.loads(mp4.get("\xa9too")[0])

            self.logger.debug("Found payload: %s", payload)

            key_url = payload["public-key"]
            signature = payload["signature"]

        except (ValueError, TypeError, IndexError, json.JSONDecodeError):
            self.logger.error("Invalid format, or no signature found")
            raise UnparseableFileError()

        return self.verify_signature(key_url, signature)



import os
from hashlib import md5
from unittest import mock

from cryptography.exceptions import InvalidSignature
import mutagen

from aletheia.exceptions import UnparseableFileError
from aletheia.file_types import Mp4File

from ...base import TestCase


class Mp4TestCase(TestCase):

    def test_get_raw_data_from_path(self):
        unsigned = os.path.join(self.DATA, "test.mp4")
        self.assertEqual(
            md5(Mp4File(unsigned, "").get_raw_data().read()).hexdigest(),
            "b9b28e4ac500be961bd07290a34cf93f"
        )

    def test_sign_from_path(self):

        path = self.copy_for_work("test.mp4")

        f = Mp4File(path, "")
        f.generate_signature = mock.Mock(return_value="signature")
        f.generate_payload = mock.Mock(return_value="payload")
        f.sign(None, None)

        mp4 = mutagen.File(path)
        self.assertEqual(mp4["\xa9too"], ["payload"])

    def test_verify_from_path_no_signature(self):

        path = self.copy_for_work("test.mp4")

        f = Mp4File(path, "")
        self.assertRaises(UnparseableFileError, f.verify)

    def test_verify_bad_signature(self):
        cache = self.cache_public_key()
        path = self.copy_for_work("test-bad-signature.mp4")

        f = Mp4File(path, cache)
        self.assertRaises(InvalidSignature, f.verify)

    def test_verify_broken_signature(self):
        cache = self.cache_public_key()
        path = self.copy_for_work("test-broken-signature.mp4")

        f = Mp4File(path, cache)
        self.assertRaises(InvalidSignature, f.verify)

    def test_verify_from_path(self):

        path = self.copy_for_work("test-signed.mp4")

        f = Mp4File(path, "")
        f.verify_signature = mock.Mock(return_value=True)
        self.assertTrue(f.verify())



import os
import sys
from os.path import abspath, dirname, join
from setuptools import setup

from aletheia import __version__

# Allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

# Shortcut for "publish" courtesy of the requests library
if sys.argv[-1] == "publish":
    os.system("rm -vr build dist")
    os.system("python setup.py build")
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    sys.exit()


# Get proper long description for package
with open(join(dirname(abspath(__file__)), "README.rst")) as f:
    description = f.read()

# Get the long description from README.md
setup(
    name="aletheia",
    version=".".join([str(_) for _ in __version__]),
    packages=[
        "aletheia",
        "aletheia.file_types"
    ],
    include_package_data=True,
    license="AGPLv3",
    description="A Python implementation of Aletheia",
    long_description=description,
    url="https://github.com/danielquinn/pyletheia",
    download_url="https://github.com/danielquinn/pyletheia",
    author="Daniel Quinn",
    author_email="code@danielquinn.org",
    maintainer="Daniel Quinn",
    maintainer_email="code@danielquinn.org",
    install_requires=[
        "Pillow>=5.0.0",
        "cryptography>=2.1.3",
        "mutagen>=1.40.0",
        "piexif>=1.0.13",
        "python-magic>=0.4.15",
        "requests>=2.18.4",
        "six",
        "termcolor>=1.1.0",
    ],
    tests_require=[
        "pytest",
        "pytest-sugar"
    ],
    extras_require={
        "doc": ["sphinx", "sphinx_rtd_theme"],
    },
    test_suite="pytest",
    entry_points={
        "console_scripts": [
            "aletheia=aletheia.cli:Command.run"
        ]
    },
    keywords=["Command Line", "verification", "fake news"],
    classifiers=[
        "Operating System :: POSIX",
        "Operating System :: Unix",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Internet :: WWW/HTTP",
    ],
)



import json

from mutagen.id3 import ID3, TPUB

from ...exceptions import UnparseableFileError
from ..base import FFMpegFile


class Mp3File(FFMpegFile):

    SCHEMA_VERSION = 1
    SUPPORTED_TYPES = ("audio/mpeg",)

    def sign(self, private_key, public_key_url):

        signature = self.generate_signature(private_key)

        self.logger.debug("Signature generated: %s", signature)

        payload = self.generate_payload(public_key_url, signature)

        mp3 = ID3(self.source)
        mp3.add(TPUB(encoding=3, text=payload))
        mp3.save()

    def verify(self):

        mp3 = ID3(self.source)

        try:

            payload = json.loads(mp3.get("TPUB")[0])

            self.logger.debug("Found payload: %s", payload)

            key_url = payload["public-key"]
            signature = payload["signature"]

        except (ValueError, TypeError, IndexError, json.JSONDecodeError):
            self.logger.error("Invalid format, or no signature found")
            raise UnparseableFileError()

        return self.verify_signature(key_url, signature)
